// UNC.cpp: implementation of the CUNC class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "UNC.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CUNC::CUNC() {

	}

CUNC::~CUNC() {

	}


//////////////////////////////////////////////////////////////////////
// Public
//////////////////////////////////////////////////////////////////////

bool CUNC::GetUNCFromLocalPath(LPCTSTR localPath, CString *uncPath) {
	bool ret = false;
	CString local = localPath;

	uncPath->Empty();

	if (local.Left(2) == "\\\\") {
		*uncPath = local;
		ret = true;  // It's UNC already
		}
	else {
		if (SearchUNCForPath(local, uncPath))
			ret = true;
		}

	return ret;
	}



//////////////////////////////////////////////////////////////////////
// Internal Methods
//////////////////////////////////////////////////////////////////////

#define HOST_NAME_SIZE	128
bool CUNC::GetLocalHostName(CString *hostName) {
	bool ret = false;

	char aux[HOST_NAME_SIZE];
	if (gethostname(aux, HOST_NAME_SIZE) == 0) { // ok
		*hostName	= aux;
		ret			= true;
		}
	else {
		int error = WSAGetLastError(); 
		if (error == WSANOTINITIALISED) {
			// The sockets subsystem must be initialized. Try
			// AfxSocketInit() in your InitInstance() or use the
			// WSAStartup - WSACleanup sequence
			return false;
			}
		}

	return ret;
	}

bool CUNC::IsNT() {
	OSVERSIONINFO	ovi;
	ovi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
	::GetVersionEx(&ovi);
	return ovi.dwPlatformId == VER_PLATFORM_WIN32_NT ? true : false;
	}

bool CUNC::BuildInitialNR(NETRESOURCE *nr) {
	bool ret = false;
	CString hostName;

	if (!GetLocalHostName(&hostName)) goto EndProc;
	hostName = "\\\\" + hostName;

	nr->dwScope			= RESOURCE_GLOBALNET;
	nr->dwType			= RESOURCETYPE_DISK;
	nr->dwDisplayType	= RESOURCEDISPLAYTYPE_GENERIC ;
	nr->dwUsage			= RESOURCEUSAGE_CONTAINER;
	nr->lpLocalName		= NULL;
	nr->lpRemoteName	= new char[hostName.GetLength() + 1];
	strcpy(nr->lpRemoteName, hostName);
	nr->lpComment		= "";
	nr->lpProvider		= NULL; // "Microsoft Network";

	ret = true;

EndProc:
	return ret;
	}


void CUNC::Get9xResourcePath(CString &resName, CString *pathResource) {
	HKEY key;
	DWORD type = REG_SZ, dataSize = 500;
	CString regPath;
	char buffer[500];

	regPath  = "Software\\Microsoft\\Windows\\CurrentVersion\\Network\\LanMan\\";
	regPath += resName;
	if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, regPath, 0, KEY_READ, &key) !=  ERROR_SUCCESS)
		return;

	if (RegQueryValueEx(key, "Path", NULL, &type, (BYTE*)buffer, &dataSize) ==  ERROR_SUCCESS) {
		*pathResource = buffer;
		//TRACE("val: %s (%s)\n", buffer, *pathResource);
		}

	RegCloseKey(key);
	}

void CUNC::GetNTResourcePath(CString &resName, CString *pathResource) {
	HKEY key;
	CString regPath, strAux;
	DWORD type, dataSize;
	
	char *buffer = NULL, *aux;
	regPath = "SYSTEM\\CurrentControlSet\\Services\\LanmanServer\\Shares";
	if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, regPath, 0, KEY_READ, &key) !=  ERROR_SUCCESS)
		return;

	if (RegQueryValueEx(key, resName, NULL, &type, NULL, &dataSize) ==  ERROR_SUCCESS) {
		buffer = new char[dataSize];
		if (buffer != NULL && RegQueryValueEx(key, resName, NULL, &type, (BYTE*)buffer, &dataSize) ==  ERROR_SUCCESS) {
			// Parse the binary information, concatened strings with double null
			bool found = false;
			aux	= buffer;  // MaxUsers = ....0Path=....0 .... 00
			while (!found) {
				strAux	= aux;  // strAux -> MaxUsers...
				if (strAux.Left(4) == "Path") {
					*pathResource = strAux.Mid(5);
					found = true;
					}
				else {
					while (*aux != 0) aux++;
					aux ++;
					if (*aux == 0) {  // double null: end of data
						return; // Not found!?
						}
					}
				}
			}
		}
	RegCloseKey(key);
	if (buffer) delete [] buffer;
	}

// Look for this resource path in the registry. The key is different in 
// Win9x and NT. Not tested in Millenium or 2000
// The key or value name relates with resource name
void CUNC::GetResourcePath(NETRESOURCE *nr, CString *pathResource) {
	pathResource->Empty();
	CString resName = nr->lpRemoteName;  //  \\server\shared
	resName			= resName.Mid(2);  // remove \\ 
	int slash		= resName.Find('\\');
	//ASSERT(slash > 0);
	resName			= resName.Mid(slash + 1);  // remove server\ 

	if (IsNT()) {
		GetNTResourcePath(resName, pathResource);
		}
	else {
		Get9xResourcePath(resName, pathResource);
		}
	}

bool CUNC::TryResource(NETRESOURCE *nr, CString &localPath, CString *pathResource) {
	GetResourcePath(nr, pathResource);	
	pathResource->MakeLower();  // c:\shared
	localPath	 .MakeLower();  // c:\shared\foo\....

	return localPath.Find(*pathResource) == 0;
	}

void CUNC::SetUNCPath(NETRESOURCE *nr, CString &localPath, CString &pathResource, CString *uncPath) {
	*uncPath	= nr->lpRemoteName;  // \\server\sharedname
	localPath	= localPath.Mid(pathResource.GetLength());  // remove common part

	if (localPath.GetAt(0) != '\\')
		*uncPath += '\\';

	*uncPath += localPath;
	}

bool CUNC::SearchUNCForPath(CString &localPath, CString *uncPath) {
	bool ret = false;
	CString pathResource;
	HANDLE hEnum = 0;
	DWORD dwBuffer = 16384 ;		// 16K is reasonable size
	DWORD dwEntries = 0xFFFFFFFF ;	// enumerate all possible entries
	LPNETRESOURCE lpnrLocal = NULL;
	NETRESOURCE initialNR;

	initialNR.lpRemoteName = NULL;
	if (!BuildInitialNR(&initialNR)) goto EndProc;

	if (WNetOpenEnum(RESOURCE_GLOBALNET, RESOURCETYPE_DISK, 0, &initialNR, 
					&hEnum) != NO_ERROR) {
		//TRACE("Error WNetOpenEnum (%s)\n", initialNR.lpRemoteName); 
		goto EndProc;
		}

	lpnrLocal = (LPNETRESOURCE) GlobalAlloc(GPTR, dwBuffer) ;

	if (WNetEnumResource(hEnum, &dwEntries, lpnrLocal, &dwBuffer) == NO_ERROR) {
		for (DWORD i=0; i<dwEntries; i++) {
			if (TryResource(&lpnrLocal[i], localPath, &pathResource)) {
				SetUNCPath(&lpnrLocal[i], localPath, pathResource, uncPath);
				ret = true;
				break;
				}
			}
		}

	if (lpnrLocal) GlobalFree((HGLOBAL) lpnrLocal) ;
	WNetCloseEnum(hEnum) ;

EndProc:
	if (initialNR.lpRemoteName) delete [] initialNR.lpRemoteName;
	return ret;
	}

bool CUNC::GetLocalPathFromUNC(CString &uncPath, CString *localPath)
{
	if(uncPath.Left(2) != "\\\\") 
	{
		*localPath = uncPath;
		return true;
	}

	int nPos = uncPath.Find('\\', 2);
	nPos = uncPath.Find('\\', nPos + 1);

	if(nPos < 0) nPos = uncPath.GetLength();

	CString path = uncPath.Right(uncPath.GetLength() - nPos);
	CString machine = uncPath.Left(nPos);

	NETRESOURCE netRes;
	netRes.lpRemoteName = (LPTSTR) (LPCTSTR) machine;

	CString local;

	GetResourcePath(&netRes, &local);

	*localPath = local + path;

	return true;
}

